# EXAMPLE 1 : http://stats.stackexchange.com/questions/72251/an-example-lasso-regression-using-glmnet-for-binary-outcome


library(glmnet)

age     <- c(4, 8, 7, 12, 6, 9, 10, 14, 7) 
gender  <- as.factor(c(1, 0, 1, 1, 1, 0, 1, 0, 0))
bmi_p   <- c(0.86, 0.45, 0.99, 0.84, 0.85, 0.67, 0.91, 0.29, 0.88) 
m_edu   <- as.factor(c(0, 1, 1, 2, 2, 3, 2, 0, 1))
p_edu   <- as.factor(c(0, 2, 2, 2, 2, 3, 2, 0, 0))
f_color <- as.factor(c("blue", "blue", "yellow", "red", "red", "yellow", 
                       "yellow", "red", "yellow"))
asthma <- c(1, 1, 0, 1, 0, 0, 0, 1, 1)

xfactors <- model.matrix(asthma ~ gender + m_edu + p_edu + f_color)[, -1]
x        <- as.matrix(data.frame(age, bmi_p, xfactors))

# Note alpha=1 for lasso only and can blend with ridge penalty down to
# alpha=0 ridge only.
glmmod <- glmnet(x, y=as.factor(asthma), alpha=1, family="binomial")

# Plot variable coefficients vs. shrinkage parameter lambda.
plot(glmmod, xvar="lambda")

cv.glmmod <- cv.glmnet(x, y=asthma, alpha=1)
plot(cv.glmmod)

(best.lambda <- cv.glmmod$lambda.min)




##########################

# the rest is from here: https://web.stanford.edu/~hastie/glmnet/glmnet_alpha.html

setwd("/Users/lukaslaffers/Google Drive/umb/dm seminar/2017/LASSO uvod")

load("QuickStartExample.RData")
fit = glmnet(x, y)
plot(fit)
print(fit)

coef(fit,s=0.1)

cvfit = cv.glmnet(x, y)
plot(cvfit)

cvfit$lambda.min

predict(cvfit, newx = x[1:5,], s = "lambda.min")

# EXAMPLE Gaussian Linear Regression
fit = glmnet(x, y, alpha = 0.2, weights = c(rep(1,50),rep(2,50)), nlambda = 20)
plot(fit, xvar = "lambda", label = TRUE)
plot(fit, xvar = "dev", label = TRUE)
predict(fit, newx = x[1:5,], type = "response", s = 0.05)
cvfit = cv.glmnet(x, y, type.measure = "mse", nfolds = 20)

#use parallelization
#install.packages("doMC")
require(doMC)
registerDoMC(cores=4)
X = matrix(rnorm(1e4 * 200), 1e4, 200)
Y = rnorm(1e4)

system.time(cv.glmnet(X, Y))
system.time(cv.glmnet(X, Y, parallel = TRUE))


# EXAMPLE with sparse matrices
load("SparseExample.RData")
fit = glmnet(x, y)
cvfit = cv.glmnet(x, y)
plot(cvfit)

i = sample(1:5, size = 25, replace = TRUE)
j = sample(1:20, size = 25, replace = TRUE)
x = rnorm(25)
nx = sparseMatrix(i = i, j = j, x = x, dims = c(5, 20))
predict(cvfit, newx = nx, s = "lambda.min")







